using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Microsoft.HomeServer.Extensibility;
using TvControl;
using TvDatabase;

namespace Microsoft.HomeServer.HomeServerConsoleTab.TVEConsole
{ 

    public partial class MainTabUserControl : UserControl
    {
        ServerInterface serverIntf;
        ExternalPlayer extPlayer;
        IConsoleServices consoleServices;

        public MainTabUserControl()
        {
            serverIntf = new ServerInterface();
            extPlayer = new ExternalPlayer();
            ClientSettings.Load();
            InitializeComponent();
        }

        public MainTabUserControl(int width, int height, IConsoleServices consoleServices)
            : this()
        {
            this.Width = width;
            this.Height = height;
            this.consoleServices = consoleServices;
        }

        private void SetDisconected()
        {
            tmrRefresh.Enabled = false;
            extPlayer.Stop();
            serverIntf.ResetConnection();
            StBarLabel.Text = "Not connected.";
            StBar.Update();
            MessageBox.Show(serverIntf.lastException.ToString(), "Exception raised", MessageBoxButtons.OK, MessageBoxIcon.Error);
            btnConnect.Visible = true;
        }

        private void serverConnectionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ServerConnectionConfig frm = new ServerConnectionConfig();
            frm.InitForm(ClientSettings.serverHostname);
            if (frm.ShowDialog() == DialogResult.OK)
            {
                frm.GetConfig(ref ClientSettings.serverHostname);
                ClientSettings.Save();
            }
        }

        private void extrenalPlayerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ExternalPlayerConfig frm = new ExternalPlayerConfig();
            frm.InitForm(ClientSettings.playerPath, ClientSettings.playerArgs);
            if (frm.ShowDialog() == DialogResult.OK)
            {
                frm.GetConfig(ref ClientSettings.playerPath, ref ClientSettings.playerArgs);
                ClientSettings.Save();
            }
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            if (!ClientSettings.IsValid())
            {
                MessageBox.Show("The settings are invalid.\nPlease check the configuration.", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            StBarLabel.Text = "Connecting to TvServer and loading groups/channels...";
            StBar.Update();
            serverIntf.ResetConnection();
            if (!serverIntf.Connect(ClientSettings.serverHostname))
            {
                SetDisconected();
                return;
            }
            cbGroups.Items.Clear();
            List<string> groups = serverIntf.GetGroupNames();
            if (groups == null)
            {
                SetDisconected();
                return;
            }
            foreach (string group in groups)
                cbGroups.Items.Add(group);
            if (cbGroups.Items.Count > 0)
                cbGroups.SelectedIndex = 0;
            StBarLabel.Text = "";
            tmrRefresh.Enabled = true;
            btnConnect.Visible = false;
        }

        private void tmrRefresh_Tick(object sender, EventArgs e)
        {
            if (!extPlayer.IsRunning())
                serverIntf.StopTimeShifting();
            ReceptionDetails recDetails = serverIntf.GetReceptionDetails();
            if (recDetails == null)
            {
                SetDisconected();
                return;
            }
            prLevel.Value = recDetails.signalLevel;
            prQuality.Value = recDetails.signalQuality;
            List<StreamingStatus> statusList = serverIntf.GetStreamingStatus();
            if (statusList == null)
            {
                SetDisconected();
                return;
            }
            lvStatus.Items.Clear();
            foreach (StreamingStatus sstate in statusList)
            {
                ListViewItem item = lvStatus.Items.Add(sstate.cardId.ToString());
                item.SubItems.Add(sstate.cardName);
                item.SubItems.Add(sstate.cardType);
                item.SubItems.Add(sstate.status);
                item.SubItems.Add(sstate.channelName);
                item.SubItems.Add(sstate.userName);
            }
            lvStatus.AutoResizeColumns(ColumnHeaderAutoResizeStyle.HeaderSize);
        }

        private void cbGroups_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateTVChannels();
        }
        private void UpdateTVChannels()
        {
            if (cbGroups.SelectedIndex == -1)
                return;
            StBarLabel.Text = "Loading referenced channels...";
            StBar.Update();
            List<ChannelInfo> refChannelInfos = serverIntf.GetChannelInfosForGroup(cbGroups.SelectedItem.ToString());
            if (refChannelInfos == null)
                SetDisconected();
            gridTVChannels.Rows.Clear();
            foreach (ChannelInfo chanInfo in refChannelInfos)
                gridTVChannels.Rows.Add(chanInfo.channelID, chanInfo.name, chanInfo.epgNow.timeInfo + "\n" + chanInfo.epgNext.timeInfo, chanInfo.epgNow.description + "\n" + chanInfo.epgNext.description);
            gridTVChannels.AutoResizeColumns();
            StBarLabel.Text = "";
        }

        private void gridTVChannels_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            StBarLabel.Text = "Trying to start timeshifting...";
            StBar.Update();
            serverIntf.StopTimeShifting();
            extPlayer.Stop();
            string rtspURL = "";
            TvResult result = serverIntf.StartTimeShifting(int.Parse(gridTVChannels.SelectedRows[0].Cells[0].Value.ToString()), ref rtspURL);
            StBarLabel.Text = "";
            StBar.Update();
            if (result != TvResult.Succeeded)
                MessageBox.Show("Could not start timeshifting\nReason: " + result.ToString());
            else
            {
                string args = string.Format(ClientSettings.playerArgs, rtspURL);
                if (!extPlayer.Start(ClientSettings.playerPath, args))
                    MessageBox.Show("Failed to start external player.", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        
    }
}
